// Copyright 2010
// Author: Li Zhang
#ifndef ZUKERALGORITHM_H_
#define ZUKERALGORITHM_H_

#include <gtest/gtest.h>
#include <sys/time.h>
#include <string>
#include <vector>
#include <utility>

using std::string;
using std::vector;

class ZukerAlgorithm
{
 public:
  // compute the structure .
  FRIEND_TEST(ZukerAlgorithmTest, computeStructure);
  void computeStructure(const std::string& input,
      std::vector<std::pair<int, int> >* result);

  // Initialize the zuker algorithm.
  FRIEND_TEST(ZukerAlgorithmTest, initZukerAlgorithm);
  void initZukerAlgorithm(const std::string& input);

  // Print the Matrices.
  void printMatrix() const;

  // Print the corresponding structure.
  void printStructure(const std::vector<std::pair<int, int> >&result) const;

  // Calculate the running time of strings and write to a file.
  FRIEND_TEST(ZukerAlgorithmTest, performance);
  void performance(const std::string& input);
 private:
  // Energy function for Stacking loop.
  FRIEND_TEST(ZukerAlgorithmTest, _eS);
  float _eS(char x, char y);
  // Energy function for Hairpin loop.
  FRIEND_TEST(ZukerAlgorithmTest, _eH);
  float _eH(char x, char y);
  // Energy function for Bulge und Interior loop.
  FRIEND_TEST(ZukerAlgorithmTest, _eBI);
  float _eBI(char i, char j, char x, char y);

  // trace back the matrix W to get the best structure.
  FRIEND_TEST(ZukerAlgorithmTest, traceBackW);
  void traceBackW(int i, int j,
      std::vector<std::pair<int, int> >* result);

  // Method for calculating the minimum energy of the given string.
  void computeW();
  FRIEND_TEST(ZukerAlgorithmTest, computeW);

  // Calculate the values of v.
  FRIEND_TEST(ZukerAlgorithmTest, computeV);
  float computeV(int, int);

  // Method for calculating the interior and bulge loop.
  FRIEND_TEST(ZukerAlgorithmTest, VBI);
  void VBI();

  // Method for tracing back the matrix V.
  FRIEND_TEST(ZukerAlgorithmTest, traceBackV);
  void traceBackV(int, int,
      std::vector<std::pair<int, int> >* result);

  // Declare the matrices and the constants.
  string _seqString;
  vector< vector<float> > _W;
  vector< vector<float> > _V;
  std::vector<float> _dictionary;
  vector< vector<float> > _traceBackW;
  vector< vector<float> > _traceBackV;
  float a;
};
#endif  // ZUKERALGORITHM_H_
