// Copyright 2010
// Author: Li Zhang

#include <gtest/gtest.h>
#include <vector>
#include <utility>
#include <string>
#include "./ZukerAlgorithm.h"

// Test for the method eS.
TEST(ZukerAlgorithmTest, _eS)
{
  ZukerAlgorithm z;
  {
    std::vector<float> result;
    result.push_back(z._eS('g', 'c'));
    result.push_back(z._eS('a', 'u'));
    result.push_back(z._eS('u', 'g'));
    result.push_back(z._eS('c', 'g'));
    result.push_back(z._eS('u', 'a'));
    result.push_back(z._eS('g', 'u'));
    result.push_back(z._eS('a', 'g'));
    ASSERT_FLOAT_EQ(result[0], -12.0);
    ASSERT_FLOAT_EQ(result[1], -12.1);
    ASSERT_FLOAT_EQ(result[2], -11.8);
    ASSERT_FLOAT_EQ(result[3], -12.0);
    ASSERT_FLOAT_EQ(result[4], -12.1);
    ASSERT_FLOAT_EQ(result[5], -11.8);
    ASSERT_FLOAT_EQ(result[6], 100);
  }
}

// Test for the method eH.
TEST(ZukerAlgorithmTest, _eH)
{
  ZukerAlgorithm z;
  {
    std::vector<float> resultTwo;
    resultTwo.push_back(z._eH('g', 'c'));
    resultTwo.push_back(z._eH('a', 'u'));
    resultTwo.push_back(z._eH('u', 'g'));
    resultTwo.push_back(z._eH('c', 'g'));
    resultTwo.push_back(z._eH('u', 'a'));
    resultTwo.push_back(z._eH('g', 'u'));
    ASSERT_FLOAT_EQ(resultTwo[0], -20.4);
    ASSERT_FLOAT_EQ(resultTwo[1], -20.7);
    ASSERT_FLOAT_EQ(resultTwo[2], -20.5);
    ASSERT_FLOAT_EQ(resultTwo[3], -20.4);
    ASSERT_FLOAT_EQ(resultTwo[4], -20.7);
    ASSERT_FLOAT_EQ(resultTwo[5], -20.5);
  }
}

// Test for the method eBI.
TEST(ZukerAlgorithmTest, _eBI)
{
  ZukerAlgorithm z;
  {
    std::vector<float> resultThree;
    resultThree.push_back(z._eBI('a', 'u', 'g', 'c'));
    resultThree.push_back(z._eBI('g', 'c', 'a', 'u'));
    resultThree.push_back(z._eBI('g', 'c', 'u', 'g'));
    resultThree.push_back(z._eBI('g', 'u', 'c', 'g'));
    resultThree.push_back(z._eBI('c', 'g', 'u', 'a'));
    resultThree.push_back(z._eBI('c', 'g', 'g', 'u'));
    ASSERT_FLOAT_EQ(resultThree[0], -8.1);
    ASSERT_FLOAT_EQ(resultThree[1], -8.25);
    ASSERT_FLOAT_EQ(resultThree[2], -8.35);
    ASSERT_FLOAT_EQ(resultThree[3], -8.05);
    ASSERT_FLOAT_EQ(resultThree[4], -8.25);
    ASSERT_FLOAT_EQ(resultThree[5], -8.35);
  }
}

// Test for the method initZukerAlgorithm.
TEST(ZukerAlgorithmTest, initZukerAlgorithm)
{
  ZukerAlgorithm z;
  // Test the string "aaa".
  z.initZukerAlgorithm("aaa");
  ASSERT_EQ(z._seqString, "aaa");
  ASSERT_EQ(z._W.size(), 3);
  ASSERT_EQ(z._W[0].size(), 3);
  ASSERT_EQ(z._W[1].size(), 3);
  ASSERT_EQ(z._W[2].size(), 3);
  ASSERT_EQ(z._V.size(), 3);
  ASSERT_EQ(z._V[0].size(), 3);
  ASSERT_EQ(z._V[1].size(), 3);
  ASSERT_EQ(z._V[2].size(), 3);
  ASSERT_EQ(z._W[0][0], 0);
  ASSERT_EQ(z._W[1][1], 0);
  ASSERT_EQ(z._W[2][1], 0);
  ASSERT_EQ(z._W[2][2], 0);
  ASSERT_EQ(z._W[1][0], 0);
  ASSERT_EQ(z._W[2][0], 0);
  ASSERT_EQ(z._W[2][1], 0);
  ASSERT_EQ(z._V[1][1], 100);
  ASSERT_EQ(z._V[2][2], 100);
  ASSERT_EQ(z._V[1][0], 100);
  ASSERT_EQ(z._V[2][1], 100);
  ASSERT_EQ(z._V[2][0], 100);
  ASSERT_EQ(z._V[2][1], 100);
  // Test the empty string.
  z.initZukerAlgorithm("");
  ASSERT_EQ(z._seqString, "");
  ASSERT_EQ(z._W.size(), 0);
  ASSERT_EQ(z._V.size(), 0);
  // Test the string aucg.
  z.initZukerAlgorithm("aucg");
  ASSERT_EQ(z._seqString, "aucg");
  ASSERT_EQ(z._W.size(), 4);
  ASSERT_EQ(z._W[0].size(), 4);
  ASSERT_EQ(z._W[1].size(), 4);
  ASSERT_EQ(z._W[2].size(), 4);
  ASSERT_EQ(z._W[3].size(), 4);
  ASSERT_EQ(z._V.size(), 4);
  ASSERT_EQ(z._V[0].size(), 4);
  ASSERT_EQ(z._V[1].size(), 4);
  ASSERT_EQ(z._V[2].size(), 4);
  ASSERT_EQ(z._V[3].size(), 4);
  ASSERT_EQ(z._W[0][0], 0);
  ASSERT_EQ(z._W[1][1], 0);
  ASSERT_EQ(z._W[2][2], 0);
  ASSERT_EQ(z._W[3][3], 0);
  ASSERT_EQ(z._W[1][0], 0);
  ASSERT_EQ(z._W[2][0], 0);
  ASSERT_EQ(z._W[3][0], 0);
  ASSERT_EQ(z._W[3][1], 0);
  ASSERT_EQ(z._W[3][2], 0);
  ASSERT_EQ(z._W[2][1], 0);
  ASSERT_EQ(z._V[0][0], 100);
  ASSERT_EQ(z._V[1][1], 100);
  ASSERT_EQ(z._V[2][2], 100);
  ASSERT_EQ(z._V[3][3], 100);
  ASSERT_EQ(z._V[1][0], 100);
  ASSERT_EQ(z._V[2][0], 100);
  ASSERT_EQ(z._V[3][0], 100);
  ASSERT_EQ(z._V[2][1], 100);
  ASSERT_EQ(z._V[3][1], 100);
  ASSERT_EQ(z._V[3][2], 100);
}

// Test for the function W.
TEST(ZukerAlgorithmTest, computeW)
{
  ZukerAlgorithm z;
  // Test the string "aucg".
  z.initZukerAlgorithm("aucg");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], -20.7);
  ASSERT_FLOAT_EQ(z._W[0][2], -20.7);
  ASSERT_FLOAT_EQ(z._W[1][2], 0);
  ASSERT_FLOAT_EQ(z._W[1][3], -20.5);
  ASSERT_FLOAT_EQ(z._W[2][3], -20.4);
  ASSERT_FLOAT_EQ(z._W[0][3], -41.1);
  // Test the string "aaa".
  z.initZukerAlgorithm("aaa");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], 0);
  ASSERT_FLOAT_EQ(z._W[0][2], 0);
  ASSERT_FLOAT_EQ(z._W[1][2], 0);
  // Test the string "au".
  z.initZukerAlgorithm("au");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], -20.7);
  // Test the string "acgu".
  z.initZukerAlgorithm("acgu");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], 0);
  ASSERT_FLOAT_EQ(z._W[0][2], -20.4);
  ASSERT_FLOAT_EQ(z._W[0][3], -32.5);
  ASSERT_FLOAT_EQ(z._W[1][2], -20.4);
  ASSERT_FLOAT_EQ(z._W[1][3], -20.5);
  ASSERT_FLOAT_EQ(z._W[2][3], -20.5);
}

// Test the function V.
TEST(ZukerAlgorithmTest, computeV)
{
  ZukerAlgorithm z;
  z.initZukerAlgorithm("aucg");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], -20.7);
  z.computeV(0, 2);
  ASSERT_FLOAT_EQ(z._V[0][2], 0);
  z.computeV(0, 3);
  ASSERT_FLOAT_EQ(z._V[0][3], -3);
  z.computeV(1, 2);
  ASSERT_FLOAT_EQ(z._V[1][2], 0);
  z.computeV(1, 3);
  ASSERT_FLOAT_EQ(z._V[1][3], -20.5);
  z.computeV(2, 3);
  ASSERT_FLOAT_EQ(z._V[2][3], -20.4);
  // Test the string "aaa".
  z.initZukerAlgorithm("aaa");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], 0);
  z.computeV(0, 2);
  ASSERT_FLOAT_EQ(z._V[0][2], 0);
  z.computeV(1, 2);
  ASSERT_FLOAT_EQ(z._V[1][2], 0);
  // Test the string "au".
  z.initZukerAlgorithm("au");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], -20.7);
  // Test the string "acgu".
  z.initZukerAlgorithm("acgu");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], 0);
  z.computeV(0, 2);
  ASSERT_FLOAT_EQ(z._V[0][2], 0);
  z.computeV(1, 2);
  ASSERT_FLOAT_EQ(z._V[1][2], -20.4);
  z.computeV(1, 3);
  ASSERT_FLOAT_EQ(z._V[1][3], 0);
  z.computeV(2, 3);
  ASSERT_FLOAT_EQ(z._V[2][3], -20.5);
}


// Test for the function traceBackW.
TEST(ZukerAlgorithmTest, traceBackW)
{
  ZukerAlgorithm z;
  z._traceBackW.resize(3);
  for (int i = 0; i < 3; i++)
  {
    z._traceBackW[i].resize(3);
  }
  z._traceBackW[0][0] = 0;
  z._traceBackW[0][1] = -3;
  z._traceBackW[0][2] = 1;
  z._traceBackW[1][0] = 0;
  z._traceBackW[1][1] = 0;
  z._traceBackW[1][2] = 0;
  z._traceBackW[2][0] = 0;
  z._traceBackW[2][1] = 0;
  z._traceBackW[2][2] = 0;
  z._traceBackV.resize(3);
  for (int i = 0; i < 3; i++)
  {
    z._traceBackV[i].resize(3);
  }
  z._traceBackV[0][0] = 0;
  z._traceBackV[0][1] = -1;
  z._traceBackV[0][2] = 0;
  z._traceBackV[1][0] = 0;
  z._traceBackV[1][1] = 0;
  z._traceBackV[1][2] = 0;
  z._traceBackV[2][0] = 0;
  z._traceBackV[2][1] = 0;
  z._traceBackV[2][2] = 0;
  vector< std::pair<int, int> > result1;
  z.traceBackW(0, 2, &result1);
  ASSERT_EQ(result1.size(), 1);
  ASSERT_EQ(result1[0].first, 0);
  ASSERT_EQ(result1[0].second, 1);
  z._traceBackW.resize(3);
  for (int i = 0; i < 3; i++)
  {
    z._traceBackW[i].resize(3);
  }
  z._traceBackW[0][0] = 0;
  z._traceBackW[0][1] = -3;
  z._traceBackW[0][2] = 1;
  z._traceBackW[1][0] = 0;
  z._traceBackW[1][1] = 0;
  z._traceBackW[1][2] = 0;
  z._traceBackW[2][0] = 0;
  z._traceBackW[2][1] = 0;
  z._traceBackW[2][2] = 0;
  for (int i = 0; i < 3; i++)
  {
    z._traceBackV[i].resize(3);
  }
  z._traceBackV[0][0] = 0;
  z._traceBackV[0][1] = -2;
  z._traceBackV[0][2] = 0;
  z._traceBackV[1][0] = -1;
  z._traceBackV[1][1] = 0;
  z._traceBackV[1][2] = 0;
  z._traceBackV[2][0] = 0;
  z._traceBackV[2][1] = 0;
  z._traceBackV[2][2] = 0;
  vector< std::pair<int, int> > result2;
  z.traceBackW(0, 2, &result2);
  ASSERT_EQ(result2.size(), 2);
  ASSERT_EQ(result2[0].first, 1);
  ASSERT_EQ(result2[0].second, 0);
  ASSERT_EQ(result2[1].first, 0);
  ASSERT_EQ(result2[1].second, 1);
}


// Test for the function traceBackV.
TEST(ZukerAlgorithmTest, traceBackV)
{
  ZukerAlgorithm z;
  // Test the matrix, which the entries of it are all 0;
  z._traceBackV.resize(2);
  for (int i = 0; i < 2; i++)
  {
    z._traceBackV[i].resize(2);
  }
  z._traceBackV[0][0] = 0;
  z._traceBackV[0][1] = -1;
  z._traceBackV[1][0] = 0;
  z._traceBackV[1][1] = 0;
  vector< std::pair<int, int> > result1;
  z.traceBackV(0, 1, &result1);
  ASSERT_EQ(result1.size(), 1);
  ASSERT_EQ(result1[0].first, 0);
  ASSERT_EQ(result1[0].second, 1);
  // Test the matrix with 9 entries.
  z._traceBackV.resize(3);
  for (int i = 0; i < 3; i++)
  {
    z._traceBackV[i].resize(3);
  }
  z._traceBackV[0][0] = 0;
  z._traceBackV[0][1] = 0;
  z._traceBackV[0][2] = -2;
  z._traceBackV[1][0] = 0;
  z._traceBackV[1][1] = -1;
  z._traceBackV[1][2] = 0;
  z._traceBackV[2][0] = 0;
  z._traceBackV[2][1] = 0;
  z._traceBackV[2][2] = 0;
  vector< std::pair<int, int> > result3;
  z.traceBackV(0, 2, &result3);
  ASSERT_EQ(result3.size(), 2);
  ASSERT_EQ(result3[0].first, 1);
  ASSERT_EQ(result3[0].second, 1);
  ASSERT_EQ(result3[1].first, 0);
  ASSERT_EQ(result3[1].second, 2);
  // Test the matrix with 9 entries, in which two of them are not 0.
  z._traceBackV.resize(4);
  for (int i = 0; i < 4; i++)
  {
    z._traceBackV[i].resize(4);
  }
  z._traceBackV[0][0] = 0;
  z._traceBackV[0][1] = 0;
  z._traceBackV[0][2] = 0;
  z._traceBackV[0][3] = 1;
  z._traceBackV[1][0] = 0;
  z._traceBackV[1][1] = -1;
  z._traceBackV[1][2] = 0;
  z._traceBackV[1][3] = 0;
  z._traceBackV[2][0] = 0;
  z._traceBackV[2][1] = 0;
  z._traceBackV[2][2] = -1;
  z._traceBackV[2][3] = 0;
  z._traceBackV[3][0] = 0;
  z._traceBackV[3][1] = 0;
  z._traceBackV[3][2] = 0;
  z._traceBackV[3][3] = 0;
  z._traceBackW.resize(4);
  for (int i = 0; i < 4; i++)
  {
    z._traceBackW[i].resize(4);
  }
  z._traceBackW[0][0] = 0;
  z._traceBackW[0][1] = 0;
  z._traceBackW[0][2] = 0;
  z._traceBackW[0][3] = 0;
  z._traceBackW[1][0] = 0;
  z._traceBackW[1][1] = -3;
  z._traceBackW[1][2] = 0;
  z._traceBackW[1][3] = 0;
  z._traceBackW[2][0] = 0;
  z._traceBackW[2][1] = 0;
  z._traceBackW[2][2] = -3;
  z._traceBackW[2][3] = 0;
  z._traceBackW[3][0] = 0;
  z._traceBackW[3][1] = 0;
  z._traceBackW[3][2] = 0;
  z._traceBackW[3][3] = 0;
  vector< std::pair<int, int> > result4;
  z.traceBackV(0, 3, &result4);
  ASSERT_EQ(result4.size(), 2);
  ASSERT_EQ(result4[0].first, 1);
  ASSERT_EQ(result4[0].second, 1);
  ASSERT_EQ(result4[1].first, 2);
  ASSERT_EQ(result4[1].second, 2);
}


// Test for the function computeStructure.
TEST(ZukerAlgorithmTest, computeStructure)
{
  ZukerAlgorithm z;
  vector< std::pair<int, int> > result;
  z.computeStructure("aucg", &result);
  ASSERT_EQ(z._W.size(), 4);
  ASSERT_FLOAT_EQ(z._W[0][0], 0);
  ASSERT_FLOAT_EQ(z._W[0][1], -20.7);
  ASSERT_FLOAT_EQ(z._W[0][2], -20.7);
  ASSERT_FLOAT_EQ(z._W[0][3], -41.1);
  ASSERT_FLOAT_EQ(z._W[1][0], 0);
  ASSERT_FLOAT_EQ(z._W[1][1], 0);
  ASSERT_FLOAT_EQ(z._W[1][2], 0);
  ASSERT_FLOAT_EQ(z._W[1][3], -20.5);
  ASSERT_FLOAT_EQ(z._W[2][0], 0);
  ASSERT_FLOAT_EQ(z._W[2][1], 0);
  ASSERT_FLOAT_EQ(z._W[2][2], 0);
  ASSERT_FLOAT_EQ(z._W[2][3], -20.4);
  ASSERT_FLOAT_EQ(z._W[3][0], 0);
  ASSERT_FLOAT_EQ(z._W[3][1], 0);
  ASSERT_FLOAT_EQ(z._W[3][2], 0);
  ASSERT_FLOAT_EQ(z._W[3][3], 0);
  ASSERT_EQ(z._V.size(), 4);
  ASSERT_FLOAT_EQ(z._V[0][0], 100);
  ASSERT_FLOAT_EQ(z._V[0][1], -20.7);
  ASSERT_FLOAT_EQ(z._V[0][2], 0);
  ASSERT_FLOAT_EQ(z._V[0][3], -3);
  ASSERT_FLOAT_EQ(z._V[1][0], 100);
  ASSERT_FLOAT_EQ(z._V[1][1], 100);
  ASSERT_FLOAT_EQ(z._V[1][2], 0);
  ASSERT_FLOAT_EQ(z._V[1][3], -20.5);
  ASSERT_FLOAT_EQ(z._V[2][0], 100);
  ASSERT_FLOAT_EQ(z._V[2][1], 100);
  ASSERT_FLOAT_EQ(z._V[2][2], 100);
  ASSERT_FLOAT_EQ(z._V[2][3], -20.4);
  ASSERT_FLOAT_EQ(z._V[3][0], 100);
  ASSERT_FLOAT_EQ(z._V[3][1], 100);
  ASSERT_FLOAT_EQ(z._V[3][2], 100);
  ASSERT_FLOAT_EQ(z._V[3][3], 100);
  ASSERT_EQ(z._traceBackW.size(), 4);
  ASSERT_FLOAT_EQ(z._traceBackW[0][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[0][1], -3);
  ASSERT_FLOAT_EQ(z._traceBackW[0][2], -1);
  ASSERT_FLOAT_EQ(z._traceBackW[0][3], 1);
  ASSERT_FLOAT_EQ(z._traceBackW[1][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[1][1], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[1][2], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[1][3], -3);
  ASSERT_FLOAT_EQ(z._traceBackW[2][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[2][1], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[2][2], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[2][3], -3);
  ASSERT_FLOAT_EQ(z._traceBackW[3][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[3][1], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[3][2], 0);
  ASSERT_FLOAT_EQ(z._traceBackW[3][3], 0);
  ASSERT_EQ(z._traceBackV.size(), 4);
  ASSERT_FLOAT_EQ(z._traceBackV[0][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[0][1], -1);
  ASSERT_FLOAT_EQ(z._traceBackV[0][2], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[0][3], 1);
  ASSERT_FLOAT_EQ(z._traceBackV[1][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[1][1], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[1][2], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[1][3], -1);
  ASSERT_FLOAT_EQ(z._traceBackV[2][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[2][1], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[2][2], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[2][3], -1);
  ASSERT_FLOAT_EQ(z._traceBackV[3][0], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[3][1], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[3][2], 0);
  ASSERT_FLOAT_EQ(z._traceBackV[3][3], 0);
}















